package com.persagy.apm.diagnose.utils;

import io.micrometer.core.instrument.util.JsonUtils;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.FileEntity;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

/**
 * @Description TODO
 * @Author feng
 * @Date 2018/7/19 12:10
 * @Version 1.0
 */
public class HttpUtils {

	private static final Logger log = LoggerFactory.getLogger(HttpUtils.class);
	private final static PoolingHttpClientConnectionManager CONNECTION_MANAGER = new PoolingHttpClientConnectionManager();
	private final static CloseableHttpClient HTTP_CLIENT;
	private static final String UTF8 = "utf-8";

	static {
		// 设置整个连接池最大连接数 根据自己的场景决定
		CONNECTION_MANAGER.setMaxTotal(200);
		// 每个路由(网站)的最大连接数
		CONNECTION_MANAGER.setDefaultMaxPerRoute(100);
		RequestConfig requestConfig = RequestConfig.custom()
				// 连接建立时间，三次握手完成时间
				.setConnectTimeout(5000)
				// 数据传输过程中数据包之间间隔的最大时间
				.setSocketTimeout(5000)
				// httpClient使用连接池来管理连接，这个时间就是从连接池获取连接的超时时间
				.setConnectionRequestTimeout(6000).build();
		HTTP_CLIENT = HttpClients.custom().setConnectionManager(CONNECTION_MANAGER)
				.setDefaultRequestConfig(requestConfig).build();
	}

	public static String httpGetRequest(String url) throws Exception {
		HttpGet httpGet = new HttpGet(url);
		return sendRequest(httpGet);
	}

	public static String httpPostRequest(String url) throws Exception {
		HttpPost httpPost = new HttpPost(url);
		return sendRequest(httpPost);
	}

	public static String httpPostRequest(String url, Map<String, Object> params) throws IOException {
		log.info("请求url：" + url);
		log.info(params.toString());
		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader("Content-Type", "application/json;charset=utf-8");
		httpPost.setEntity(new StringEntity(com.persagy.apm.diagnose.utils.JsonUtils.beanToJson(params), UTF8));
		String respContent = sendRequest(httpPost);
//		log.info(respContent);
		return respContent;
	}

	public static String httpPost(String url, Map<String, Object> params) throws IOException {
		log.info("请求url：" + url);
		log.info(params.toString());
		HttpPost httpPost = new HttpPost(url);
	//	httpPost.addHeader("Content-Type", "application/x-www-form-urlencoded");
		httpPost.setEntity(new StringEntity(com.persagy.apm.diagnose.utils.JsonUtils.beanToJson(params), UTF8));
		String respContent = sendRequest(httpPost);
//		log.info(respContent);
		return respContent;
	}

	private static String sendRequest(HttpUriRequest request) throws IOException {
		CloseableHttpResponse response = HTTP_CLIENT.execute(request);
		return EntityUtils.toString(response.getEntity(), UTF8);
	}

	public static String httpPostRequest(String url, InputStream is) throws Exception {
		HttpPost httpPost = new HttpPost(url);
		httpPost.setEntity(new InputStreamEntity(is));
		return sendRequest(httpPost);
	}

	public static String httpPostRequest(String url, File file) throws Exception {
		HttpPost httpPost = new HttpPost(url);
		httpPost.setEntity(new FileEntity(file));
		return sendRequest(httpPost);
	}

	public static byte[] httpGetFile(String url) throws Exception {
		HttpGet httpGet = new HttpGet(url);
		CloseableHttpResponse response = HTTP_CLIENT.execute(httpGet);
		return EntityUtils.toByteArray(response.getEntity());
	}

}
